@@ -2,7 +2,9 @@
 
 # region #-- imports --#
 import logging
-from typing import Dict, List, Optional, Tuple
+import re
+import uuid
+from typing import Dict, List, Tuple
 
 import aiohttp
 import asyncclick as click
@@ -12,6 +14,7 @@ from pyvelop.device import Device
 from pyvelop.exceptions import (
     MeshConnectionError,
     MeshDeviceNotFoundResponse,
+    MeshException,
     MeshInvalidCredentials,
     MeshInvalidInput,
     MeshNodeNotPrimary,
@@ -128,106 +131,159 @@ def cli() -> None:
     """CLI for interacting with the pyvelop module."""
 
 
-@cli.command(cls=StandardCommand)
+@cli.group(name="device")
+@click.help_option()
+async def device_group() -> None:
+    """Work with devices on the mesh."""
+
+
+@device_group.command(cls=StandardCommand, name="delete")
 @click.pass_context
-@click.argument("device_name")
-async def device(
+@click.argument("device")
+async def device_delete(
     ctx: click.Context,
-    device_name: str,
+    device: str,
     **_,
 ) -> None:
-    """Get details about a device on the Mesh."""
-    if mesh_details := await mesh_connect(ctx):
-        async with mesh_details:
-            device_details: List[Device] = await mesh_details.async_get_devices()
-            for found_device in device_details:
-                if found_device.name == device_name:
-                    _display_data(
-                        _build_display_data(
-                            mappings=[
-                                ("results_time", "Queried at"),
-                                ("unique_id", "Device ID"),
-                                ("ui_type", "Icon Type"),
-                                ("manufacturer", "Manufacturer"),
-                                ("model", "Model"),
-                                ("description", "Description"),
-                                ("operating_system", "Operating System"),
-                                ("serial", "Serial #"),
-                                ("status", "Online"),
-                                ("parent_name", "Parent"),
-                                (
-                                    "connected_adapters",
-                                    "Connections",
-                                    _connected_details(
-                                        adapters=found_device.connected_adapters
-                                    ),
-                                ),
-                                (
-                                    "parental_control_schedule",
-                                    "Parental Control",
-                                    _parental_control_schedule_details(
-                                        schedule=found_device.parental_control_schedule
-                                    ),
-                                ),
-                            ],
-                            obj=found_device,
-                            title=found_device.name,
+    """Delete a device on the Mesh."""
+    devices: List[Device] | None = await _get_device_details(ctx=ctx, device=device)
+
+    if devices is not None:
+        if mesh_obj := await mesh_connect(ctx):
+            async with mesh_obj:
+                for found_device in devices:
+                    try:
+                        await mesh_obj.async_delete_device_by_id(
+                            device=found_device.unique_id
                         )
-                    )
-                    break
+                    except MeshException as err:
+                        _LOGGER.error("%s (%s)", err, found_device.name)
 
 
-@cli.command(cls=StandardCommand)
+@device_group.command(cls=StandardCommand, name="details")
 @click.pass_context
-async def mesh(
+@click.argument("device")
+async def device_details(
+    ctx: click.Context,
+    device: str,
+    **_,
+) -> None:
+    """Display details about a device on the Mesh."""
+    devices: List[Device] | None = await _get_device_details(ctx=ctx, device=device)
+
+    if devices is not None:
+        for found_device in devices:
+            _display_data(
+                _build_display_data(
+                    mappings=[
+                        ("results_time", "Queried at"),
+                        ("unique_id", "Device ID"),
+                        ("ui_type", "Icon Type"),
+                        ("manufacturer", "Manufacturer"),
+                        ("model", "Model"),
+                        ("description", "Description"),
+                        ("operating_system", "Operating System"),
+                        ("serial", "Serial #"),
+                        ("status", "Online"),
+                        ("parent_name", "Parent"),
+                        (
+                            "connected_adapters",
+                            "Connections",
+                            _connected_details(
+                                adapters=found_device.connected_adapters
+                            ),
+                        ),
+                        (
+                            "parental_control_schedule",
+                            "Parental Control",
+                            _parental_control_schedule_details(
+                                schedule=found_device.parental_control_schedule
+                            ),
+                        ),
+                    ],
+                    obj=found_device,
+                    title=found_device.name,
+                )
+            )
+
+
+@cli.group(name="mesh")
+@click.help_option()
+async def mesh_group() -> None:
+    """Work with the mesh."""
+
+
+@mesh_group.command(cls=StandardCommand, name="details")
+@click.pass_context
+async def mesh_details(
     ctx: click.Context,
     **_,
 ) -> None:
     """Get details about the Mesh."""
     indent: int = DEF_INDENT
     prefix: str = f"\n{indent * ' '}"
-    if mesh_details := await mesh_connect(ctx):
-        async with mesh_details:
-            await mesh_details.async_gather_details()
+    if mesh_obj := await mesh_connect(ctx):
+        async with mesh_obj:
+            await mesh_obj.async_gather_details()
             _display_data(
                 _build_display_data(
                     mappings=[
                         ("wan_status", "Internet Connected"),
                         ("wan_ip", "Public IP"),
-                        ("wan_dns", "DNS Servers", ", ".join(mesh_details.wan_dns)),
+                        ("wan_dns", "DNS Servers", ", ".join(mesh_obj.wan_dns)),
                         ("wan_mac", "MAC"),
                         (
                             "nodes",
                             "Nodes",
                             prefix
-                            + prefix.join([node.name for node in mesh_details.nodes]),
+                            + prefix.join([node.name for node in mesh_obj.nodes]),
+                        ),
+                        (
+                            "latest_speedtest_result",
+                            "Latest Speedtest Result",
+                            _speedtest_results(
+                                speedtest_results=mesh_obj.latest_speedtest_result
+                            ),
                         ),
-                        ("latest_speedtest_result", "Latest Speedtest Result"),
                         ("parental_control_enabled", "Parental Control Enabled"),
+                        ("wps_state", "WPS Enabled"),
+                        ("homekit_enabled", "HomeKit Integration Enabled"),
+                        ("homekit_paired", "HomeKit Integration Paired"),
+                        ("client_steering_enabled", "Client Steering Enabled"),
+                        ("node_steering_enabled", "Node Steering Enabled"),
+                        (
+                            "mac_filtering",
+                            "MAC Filtering",
+                            _mac_filtering_details(
+                                addresses=mesh_obj.mac_filtering_addresses,
+                                mode=mesh_obj.mac_filtering_mode,
+                                state=mesh_obj.mac_filtering_enabled,
+                            ),
+                        ),
                         (
                             "guest_wifi_details",
                             "Guest Wi-Fi Details",
                             _guest_wifi_details(
-                                state=mesh_details.guest_wifi_enabled,
-                                networks=mesh_details.guest_wifi_details,
+                                state=mesh_obj.guest_wifi_enabled,
+                                networks=mesh_obj.guest_wifi_details,
                             ),
                         ),
                         (
                             "storage_details",
                             "Storage Details",
                             _storage_details(
-                                available_shares=mesh_details.storage_available,
-                                server_details=mesh_details.storage_settings,
+                                available_shares=mesh_obj.storage_available,
+                                server_details=mesh_obj.storage_settings,
                             ),
                         ),
                         (
                             "devices",
-                            f"Online Devices ({len([device for device in mesh_details.devices if device.status])})",
+                            f"Online Devices ({len([device for device in mesh_obj.devices if device.status])})",
                             prefix
                             + prefix.join(
                                 [
                                     f"{device.name} ({device.connected_adapters[0].get('ip')})"
-                                    for device in mesh_details.devices
+                                    for device in mesh_obj.devices
                                     if device.status
                                 ]
                             ),
@@ -235,33 +291,33 @@ async def mesh(
                         (
                             "devices",
                             "Offline Devices "
-                            f"({len([device for device in mesh_details.devices if not device.status])})",
+                            f"({len([device for device in mesh_obj.devices if not device.status])})",
                             prefix
                             + prefix.join(
                                 [
                                     device.name
-                                    for device in mesh_details.devices
+                                    for device in mesh_obj.devices
                                     if not device.status
                                 ]
                             ),
                         ),
                     ],
-                    obj=mesh_details,
+                    obj=mesh_obj,
                     title="Mesh Overview",
                 )
             )
 
 
-@cli.group()
+@cli.group(name="node")
 @click.help_option()
-async def node() -> None:
+async def node_group() -> None:
     """Work with nodes on the Mesh."""
 
 
-@node.command(cls=StandardCommand)
+@node_group.command(cls=StandardCommand, name="details")
 @click.argument("node_name")
 @click.pass_context
-async def details(
+async def node_details(
     ctx: click.Context,
     node_name: str,
     **_,
@@ -269,15 +325,15 @@ async def details(
     """Get details about a node on the Mesh."""
     indent: int = DEF_INDENT
     prefix: str = f"\n{indent * ' '}"
-    if mesh_details := await mesh_connect(ctx):
-        async with mesh_details:
-            await mesh_details.async_gather_details()
-            node_details: List[Node] = mesh_details.nodes
-            if not node_details:
+    if mesh_obj := await mesh_connect(ctx):
+        async with mesh_obj:
+            await mesh_obj.async_gather_details()
+            nodes: List[Node] = mesh_obj.nodes
+            if not nodes:
                 print("No nodes found")
             else:
                 found_node: Node
-                for found_node in node_details:
+                for found_node in nodes:
                     if found_node.name == node_name:
                         _display_data(
                             _build_display_data(
@@ -364,26 +420,26 @@ async def details(
                         break
 
 
-@node.command(cls=StandardCommand)
+@node_group.command(cls=StandardCommand, name="restart")
 @click.argument("node_name")
 @click.pass_context
-async def restart(
+async def node_restart(
     ctx: click.Context,
     node_name: str,
     **_,
 ) -> None:
     """Restart a node on the Mesh."""
-    if mesh_details := await mesh_connect(ctx):
-        async with mesh_details:
+    if mesh_obj := await mesh_connect(ctx):
+        async with mesh_obj:
             try:
-                await mesh_details.async_gather_details()
+                await mesh_obj.async_gather_details()
                 _LOGGER.debug("Restarting %s", node_name)
-                await mesh_details.async_reboot_node(node_name=node_name)
+                await mesh_obj.async_reboot_node(node_name=node_name)
             except (MeshDeviceNotFoundResponse, MeshInvalidInput) as err:
                 _LOGGER.error(err)
 
 
-async def mesh_connect(ctx: click.Context = None) -> Optional[Mesh]:
+async def mesh_connect(ctx: click.Context = None) -> Mesh | None:
     """Return the Mesh object."""
     if ctx is not None:
         mesh_object: Mesh = Mesh(
@@ -414,6 +470,48 @@ async def mesh_connect(ctx: click.Context = None) -> Optional[Mesh]:
     return None
 
 
+async def _get_device_details(ctx: click.Context, device: str) -> List[Device] | None:
+    """Retreive device details from the mesh."""
+    ret: List[Device | Node] | None
+    if mesh_obj := await mesh_connect(ctx):
+        async with mesh_obj:
+            try:  # match a GUID?
+                _ = uuid.UUID(device)
+                ret = [
+                    await mesh_obj.async_get_device_from_id(
+                        device_id=device, force_refresh=True
+                    )
+                ]
+            except MeshDeviceNotFoundResponse:
+                _LOGGER.error("Device not found (%s)", device)
+                return
+            except ValueError:  # not a GUID
+                regex_pattern: str = r"^[a-f0-9]{2}((:|-)*[a-f0-9]{2}){5}$"
+                if (  # MAC address?
+                    re.match(pattern=regex_pattern, string=device, flags=re.IGNORECASE)
+                    is not None
+                ):
+                    try:
+                        ret = [
+                            await mesh_obj.async_get_device_from_mac_address(
+                                device, force_refresh=True
+                            )
+                        ]
+                    except MeshDeviceNotFoundResponse:
+                        _LOGGER.error("Device not found (%s)", device)
+                        return
+                else:
+                    ret = [
+                        found_device
+                        for found_device in await mesh_obj.async_get_devices()
+                        if found_device.name == device
+                    ]
+                    if not ret:
+                        _LOGGER.error("Device not found (%s)", device)
+                        return
+    return ret
+
+
 def _build_display_data(
     mappings: List[Tuple],
     obj: Device | Dict | Mesh | Node,
@@ -508,6 +606,37 @@ def _guest_wifi_details(state: bool, networks: List[Dict]) -> str:
     return "\n" + ret.rstrip()
 
 
+def _mac_filtering_details(addresses: List[str], mode: str | None, state: bool) -> str:
+    """Format the MAC filtering details for display."""
+    indent: int = DEF_INDENT
+
+    ret = _build_display_data(
+        indent=indent,
+        mappings=[
+            ("state", "Enabled"),
+            ("mode", "Mode", str(mode).title()),
+            (
+                "addresses",
+                "Addresses",
+                "\n"
+                + "\n".join(
+                    [
+                        f"{indent * 2 * ' '}{idx}: {addr}"
+                        for idx, addr in enumerate(addresses)
+                    ]
+                ),
+            ),
+        ],
+        obj={
+            "addresses": addresses,
+            "mode": mode,
+            "state": state,
+        },
+    )
+
+    return "\n" + ret.rstrip()
+
+
 def _parental_control_schedule_details(schedule: Dict) -> str:
     """Format the parental control schedule for display."""
     indent: int = DEF_INDENT
@@ -544,6 +673,32 @@ def _parental_control_schedule_details(schedule: Dict) -> str:
     return "\n" + ret.rstrip()
 
 
+def _speedtest_results(speedtest_results: Dict) -> str:
+    """Format the Speedtest results for display."""
+    indent: int = DEF_INDENT
+    ret: str = _build_display_data(
+        indent=indent,
+        mappings=[
+            ("timestamp", "Executed at"),
+            (
+                "download_bandwidth",
+                "Download",
+                f"{round(speedtest_results.get('download_bandwidth', 0) / 1000, 2)} Mbps",
+            ),
+            (
+                "upload_bandwidth",
+                "Upload",
+                f"{round(speedtest_results.get('upload_bandwidth', 0) / 1000, 2)} Mbps",
+            ),
+            ("latency", "Latency", f"{speedtest_results.get('latency')}ms"),
+            ("exit_code", "Status"),
+        ],
+        obj=speedtest_results,
+    )
+
+    return "\n" + ret.rstrip()
+
+
 def _storage_details(available_shares: List[Dict], server_details: Dict) -> str:
     """Format the storage details for display."""
     ret: str = ""
