@@ -13,6 +13,14 @@ class MeshBadResponse(MeshException):
         super().__init__("Bad Response")
 
 
+class MeshCannotDeleteDevice(MeshException):
+    """Unable to delete device."""
+
+    def __init__(self) -> None:
+        """Initialise and default message."""
+        super().__init__("Unable to delete the device")
+
+
 class MeshConnectionError(MeshException):
     """Connection error for the API."""
 
