@@ -14,6 +14,7 @@ import aiohttp
 from .const import DEF_REDACT
 from .exceptions import (
     MeshBadResponse,
+    MeshCannotDeleteDevice,
     MeshConnectionError,
     MeshInvalidCredentials,
     MeshInvalidInput,
@@ -51,6 +52,8 @@ class Actions:
         f"{ROOT}/firmwareupdate/GetFirmwareUpdateSettings"
     )
     GET_GUEST_NETWORK_INFO: str = f"{ROOT}/guestnetwork/GetGuestRadioSettings2"
+    GET_HOMEKIT_SETTINGS: str = f"{ROOT}/homekit/GetHomeKitSettings"
+    GET_MAC_FILTERING_SETTINGS: str = f"{ROOT}/macfilter/GetMACFilterSettings"
     GET_NETWORK_CONNECTIONS: str = (
         f"{ROOT}/nodes/networkconnections/GetNodesWirelessNetworkConnections"
     )
@@ -61,11 +64,15 @@ class Actions:
     GET_SPEEDTEST_STATUS: str = f"{ROOT}/healthcheck/GetHealthCheckStatus"
     GET_STORAGE_PARTITIONS: str = f"{ROOT}/nodes/storage/GetNodesPartitions"
     GET_STORAGE_SMB_SERVER: str = f"{ROOT}/nodes/storage/GetSMBServerSettings"
+    GET_TOPOLOGY_OPTIMISATION_SETTINGS: str = (
+        f"{ROOT}/nodes/topologyoptimization/GetTopologyOptimizationSettings2"
+    )
     GET_UPDATE_FIRMWARE_STATE: str = (
         f"{ROOT}/nodes/firmwareupdate/GetFirmwareUpdateStatus"
     )
     GET_UPDATE_SETTINGS: str = f"{ROOT}/firmwareupdate/GetFirmwareUpdateSettings"
     GET_WAN_INFO: str = f"{ROOT}/router/GetWANStatus3"
+    GET_WPS_SERVER_SETTINGS: str = f"{ROOT}/wirelessap/GetWPSServerSettings"
     REBOOT: str = f"{ROOT}/core/Reboot"
     SET_GUEST_NETWORK: str = f"{ROOT}/guestnetwork/SetGuestRadioSettings2"
     SET_PARENTAL_CONTROL_INFO: str = (
@@ -235,7 +242,9 @@ class Response:
             err = None
             for resp in responses:
                 err = None
-                if resp.get(self.RESULT_KEY) == "_ErrorInvalidInput":
+                if resp is None:
+                    err = MeshInvalidOutput(resp)
+                elif resp.get(self.RESULT_KEY) == "_ErrorInvalidInput":
                     err = MeshInvalidInput(resp.get("error"))
                 elif resp.get(self.RESULT_KEY) == "_ErrorInvalidOutput":
                     err = MeshInvalidOutput(resp.get("error"))
@@ -248,6 +257,8 @@ class Response:
                         else f"Unknown action URI '{self.action}'"
                     )
                     err = MeshInvalidInput(action)
+                elif resp.get(self.RESULT_KEY) == "ErrorCannotDeleteDevice":
+                    err = MeshCannotDeleteDevice
                 elif resp.get(self.RESULT_KEY) == "ErrorDeviceNotInMasterMode":
                     err = MeshNodeNotPrimary
                 elif resp.get(self.RESULT_KEY).startswith("_"):
