@@ -20,6 +20,7 @@ from .exceptions import (
     MeshException,
     MeshInvalidArguments,
     MeshInvalidInput,
+    MeshInvalidOutput,
     MeshTooManyMatches,
 )
 from .logger import Logger
@@ -34,6 +35,8 @@ _LOGGER_VERBOSE = logging.getLogger(f"{__name__}.verbose")
 ATTR_BACKHAUL_INFO: str = "backhaul"
 ATTR_FIRMWARE_UPDATE_SETTINGS: str = "firmware_update_settings"
 ATTR_GUEST_NETWORK_INFO: str = "guest_network"
+ATTR_HOMEKIT_SETTINGS: str = "homekit_settings"
+ATTR_MAC_FILTERING_SETTINGS: str = "mac_filtering_settings"
 ATTR_NETWORK_CONNECTIONS: str = "network_connections"
 ATTR_NODES: str = "nodes"
 ATTR_PARENTAL_CONTROL_INFO: str = "parental_control"
@@ -43,8 +46,10 @@ ATTR_SPEEDTEST_STATUS: str = "speedtest_status"
 ATTR_RAW_DEVICES: str = "raw_devices"
 ATTR_STORAGE_PARTITIONS: str = "storage_partitions"
 ATTR_STORAGE_SMB_SERVER: str = "storage_smb_server"
+ATTR_TOPOLOGY_OPTIMISATION_SETTINGS: str = "topology_optimisation_settings"
 ATTR_UPDATE_FIRMWARE_STATE: str = "check_update_state"
 ATTR_WAN_INFO: str = "wan_info"
+ATTR_WPS_SERVER_SETTINGS: str = "wps_server_settings"
 # endregion
 
 JNAP_ACTION_TO_ATTRIBUTE: dict = {
@@ -52,13 +57,17 @@ JNAP_ACTION_TO_ATTRIBUTE: dict = {
     api.Actions.GET_DEVICES: ATTR_RAW_DEVICES,
     api.Actions.GET_FIRMWARE_UPDATE_SETTINGS: ATTR_FIRMWARE_UPDATE_SETTINGS,
     api.Actions.GET_GUEST_NETWORK_INFO: ATTR_GUEST_NETWORK_INFO,
+    api.Actions.GET_HOMEKIT_SETTINGS: ATTR_HOMEKIT_SETTINGS,
+    api.Actions.GET_MAC_FILTERING_SETTINGS: ATTR_MAC_FILTERING_SETTINGS,
     api.Actions.GET_NETWORK_CONNECTIONS: ATTR_NETWORK_CONNECTIONS,
     api.Actions.GET_PARENTAL_CONTROL_INFO: ATTR_PARENTAL_CONTROL_INFO,
     api.Actions.GET_SPEEDTEST_RESULTS: ATTR_SPEEDTEST_RESULTS,
     api.Actions.GET_SPEEDTEST_STATUS: ATTR_SPEEDTEST_STATUS,
     api.Actions.GET_STORAGE_PARTITIONS: ATTR_STORAGE_PARTITIONS,
     api.Actions.GET_STORAGE_SMB_SERVER: ATTR_STORAGE_SMB_SERVER,
+    api.Actions.GET_TOPOLOGY_OPTIMISATION_SETTINGS: ATTR_TOPOLOGY_OPTIMISATION_SETTINGS,
     api.Actions.GET_WAN_INFO: ATTR_WAN_INFO,
+    api.Actions.GET_WPS_SERVER_SETTINGS: ATTR_WPS_SERVER_SETTINGS,
     api.Actions.GET_UPDATE_FIRMWARE_STATE: ATTR_UPDATE_FIRMWARE_STATE,
 }
 
@@ -273,12 +282,16 @@ class Mesh:
         :param include_firmware_update: True to include the current firmware update details (does not issue a check)
         :param include_firmware_update_settings: True to include the current settings for firmware updates
         :param include_guest_wifi: True to include details about the guest Wi-Fi
+        :param include_homekit_settings: True to include the settings for the HomeKit integration
+        :param include_mac_filtering_settings: True to include the settings for MAC filtering
         :param include_network_connections: True to include details about network connections
         :param include_parental_control: True to include details about Parental Control
         :param include_speedtest_results: True to include the latest completed Speedtest result
         :param include_speedtest_status: True to include the currently running speedtest status
         :param include_storage: True to include the external storage details if available
+        :param include_topology_optimisation_settings: True to include details about topology optimisation
         :param include_wan: True to include WAN details
+        :param include_wps_server_settings: True to include the WPS server settings
         :return: A dictionary containing the relevant details.  Keys used will match those of the instance variable.
         """
         _LOGGER.debug(
@@ -300,10 +313,18 @@ class Mesh:
         if kwargs.get("include_backhaul") or kwargs.get("include_devices"):
             payload_safe.append({"action": api.Actions.GET_BACKHAUL})
 
+        # -- get the update check details --#
+        if kwargs.get("include_firmware_update"):
+            payload_safe.append({"action": api.Actions.GET_UPDATE_FIRMWARE_STATE})
+
         # -- get the guest WiFi details --#
         if kwargs.get("include_guest_wifi"):
             payload_safe.append({"action": api.Actions.GET_GUEST_NETWORK_INFO})
 
+        # -- get the MAC filtering settings --#
+        if kwargs.get("include_mac_filtering_settings"):
+            payload_safe.append({"action": api.Actions.GET_MAC_FILTERING_SETTINGS})
+
         # -- get the Parental Control details --#
         if kwargs.get("include_parental_control") or kwargs.get("include_devices"):
             payload_safe.append({"action": api.Actions.GET_PARENTAL_CONTROL_INFO})
@@ -324,14 +345,20 @@ class Mesh:
                 }
             )
 
-        # -- get the update check details --#
-        if kwargs.get("include_firmware_update"):
-            payload_safe.append({"action": api.Actions.GET_UPDATE_FIRMWARE_STATE})
+        # -- get the topology optimisation settings --#
+        if kwargs.get("include_topology_optimisation_settings"):
+            payload_safe.append(
+                {"action": api.Actions.GET_TOPOLOGY_OPTIMISATION_SETTINGS}
+            )
 
         # -- get the WAN details --#
         if kwargs.get("include_wan"):
             payload_safe.append({"action": api.Actions.GET_WAN_INFO})
 
+        # -- get the WPS server settings --#
+        if kwargs.get("include_wps_server_settings"):
+            payload_safe.append({"action": api.Actions.GET_WPS_SERVER_SETTINGS})
+
         request_safe = self._async_make_request(
             action=api.Actions.TRANSACTION,
             payload=_build_action_payload(transaction=payload_safe),
@@ -341,6 +368,15 @@ class Mesh:
 
         # region #-- prepare the potentially unsafe requests --#
         request_unsafe = []
+        if kwargs.get("include_homekit_settings"):
+            request_unsafe.append(
+                self._async_make_request(
+                    action=api.Actions.GET_HOMEKIT_SETTINGS,
+                    payload={},
+                    raise_on_error=False,
+                )
+            )
+
         if kwargs.get("include_network_connections"):
             request_unsafe.append(
                 self._async_make_request(
@@ -558,45 +594,42 @@ class Mesh:
             await self._session.close()
             _LOGGER.debug(self._log_formatter.format("exited"))
 
-    async def async_delete_device(self, **kwargs) -> None:
-        """Delete a device from the device list on the mesh.
+    async def async_delete_device_by_id(self, device: str) -> None:
+        """Delete a device from the device list on the mesh by its ID.
 
-        Supports deleting by device ID or device name.
-        Will error if neither the device ID nor name are given.
-        Will error if multiple devices match the given name.
-
-        :param kwargs: keyword arguments (device_id, device_name)
+        :param device: the unique id of the device to delete
         :return: None
         """
-        _LOGGER.debug(self._log_formatter.format("entered, args: %s"), kwargs)
+        _LOGGER.debug(self._log_formatter.format("entered, device: %s"), device)
 
-        device_id: str
-        if "device_id" in kwargs:
-            device_id = kwargs.get("device_id")
-        elif "device_name" in kwargs:
-            dev: Device
-            device = [
-                dev
-                for dev in self._mesh_attributes[ATTR_PROCESSED_DEVICES]
-                if dev.name == kwargs.get("device_name")
-            ]
-            if len(device) == 0:
-                raise MeshDeviceNotFoundResponse
+        await self._async_make_request(
+            action=api.Actions.DELETE_DEVICE, payload={"deviceID": device}
+        )
 
-            if len(device) > 1:
-                raise MeshTooManyMatches
+    async def async_delete_device_by_name(self, device: str) -> None:
+        """Delete a device from the device list on the mesh by name.
 
-            device_id = device[0].unique_id
-        else:
-            device_id = ""
+        Will error if multiple devices match the given name or no matching
+        device is found.
 
-        if device_id:
-            payload = {"deviceID": device_id}
-            await self._async_make_request(
-                action=api.Actions.DELETE_DEVICE, payload=payload
-            )
-        else:
-            raise MeshInvalidArguments
+        :param device: the name of the device to delete
+        :return: None
+        """
+        _LOGGER.debug(self._log_formatter.format("entered, name: %s"), device)
+
+        dev: Device
+        device = [
+            dev
+            for dev in self._mesh_attributes[ATTR_PROCESSED_DEVICES]
+            if dev.name == device
+        ]
+        if len(device) == 0:
+            raise MeshDeviceNotFoundResponse
+
+        if len(device) > 1:
+            raise MeshTooManyMatches
+
+        await self.async_delete_device_by_id(device=device[0].unique_id)
 
         _LOGGER.debug(self._log_formatter.format("exited"))
 
@@ -615,12 +648,16 @@ class Mesh:
             include_firmware_update=True,
             include_firmware_update_settings=True,
             include_guest_wifi=True,
+            include_homekit_settings=True,
+            include_mac_filtering_settings=True,
             include_network_connections=True,
             include_parental_control=True,
             include_speedtest_results=True,
             include_speedtest_status=True,
             include_storage=True,
+            include_topology_optimisation_settings=True,
             include_wan=True,
+            include_wps_server_settings=True,
         )
 
         # region #-- split the devices into their types --#
@@ -684,6 +721,9 @@ class Mesh:
             )
             all_devices = resp.get(ATTR_PROCESSED_DEVICES)
 
+        if not all_devices:
+            raise MeshInvalidOutput from None
+
         try:
             ret = [device for device in all_devices if device.unique_id == device_id][0]
         except IndexError as err:
@@ -968,6 +1008,14 @@ class Mesh:
 
         return ret
 
+    @property
+    @needs_gather_details
+    def client_steering_enabled(self) -> bool:
+        """Return if client steering is enabled."""
+        return self._mesh_attributes.get(ATTR_TOPOLOGY_OPTIMISATION_SETTINGS, {}).get(
+            "isClientSteeringEnabled", False
+        )
+
     @property
     @needs_gather_details
     def connected_node(self) -> str:
@@ -1035,6 +1083,22 @@ class Mesh:
         ]
         return ret
 
+    @property
+    @needs_gather_details
+    def homekit_enabled(self) -> bool:
+        """Return if the HomeKit integration is enabled."""
+        return self._mesh_attributes.get(ATTR_HOMEKIT_SETTINGS, {}).get(
+            "isEnabled", False
+        )
+
+    @property
+    @needs_gather_details
+    def homekit_paired(self) -> bool:
+        """Return if the HomeKit integration is paired."""
+        return self._mesh_attributes.get(ATTR_HOMEKIT_SETTINGS, {}).get(
+            "isPaired", False
+        )
+
     @property
     @needs_gather_details
     def latest_speedtest_result(self) -> Optional[Dict]:
@@ -1056,6 +1120,46 @@ class Mesh:
 
         return ret or None
 
+    @property
+    @needs_gather_details
+    def mac_filtering_addresses(self) -> List[str]:
+        """Return address that are configured for MAC filtering."""
+        return self._mesh_attributes.get(ATTR_MAC_FILTERING_SETTINGS, {}).get(
+            "macAddresses", []
+        )
+
+    @property
+    @needs_gather_details
+    def mac_filtering_enabled(self) -> bool:
+        """Return if MAC filtering is enabled."""
+        return (
+            self._mesh_attributes.get(ATTR_MAC_FILTERING_SETTINGS, {})
+            .get("macFilterMode", "")
+            .lower()
+            != "disabled"
+        )
+
+    @property
+    @needs_gather_details
+    def mac_filtering_mode(self) -> str | None:
+        """Return the MAC filtering mode."""
+        if self.mac_filtering_enabled:
+            return (
+                self._mesh_attributes.get(ATTR_MAC_FILTERING_SETTINGS, {})
+                .get("macFilterMode", "")
+                .lower()
+            )
+
+        return None
+
+    @property
+    @needs_gather_details
+    def node_steering_enabled(self) -> bool:
+        """Return if node steering is enabled."""
+        return self._mesh_attributes.get(ATTR_TOPOLOGY_OPTIMISATION_SETTINGS, {}).get(
+            "isNodeSteeringEnabled", False
+        )
+
     @property
     @needs_gather_details
     def nodes(self) -> List:
@@ -1210,4 +1314,12 @@ class Mesh:
             == "connected"
         )
 
+    @property
+    @needs_gather_details
+    def wps_state(self) -> bool:
+        """Return if WPS is enabled or not."""
+        return self._mesh_attributes.get(ATTR_WPS_SERVER_SETTINGS, {}).get(
+            "enabled", False
+        )
+
     # endregion
